#include <stdio.h>
#include <stdlib.h>
#include <stm32f0xx.h>
#include "common.h"
#include "usart.h"
#include "ik_nec.h"
#include "tm1637.h"
#include "rda5807.h"
#include "ds3231.h"

#define USART_ON

volatile uint32_t ttms           = 0; // системный тикер
volatile uint32_t _pb3led        = 0; // мигалка _pb3led задержка
volatile uint8_t  ispowerup      = 0;
volatile uint16_t ik_out         = 0; // код кнопки от ИК-пульта из прерывания
volatile uint32_t ikp            = 0; // для IR (замер паузы между посылками)
volatile uint16_t ik_out_prev    = 0; // предыдущий код Ик / пока не исп.
volatile uint8_t  ik_out_repeatf = 0; // флаг повтора

volatile uint8_t  last_volume   = 0; // тут
volatile uint16_t last_freq     = 0; //     понятно
volatile uint8_t  last_dispmode = 0; //             и так

volatile uint8_t  dispmode = 0; // 0 - freq, 1 - volume
volatile uint16_t dispcntr = 0; // задержка отображений не основного режима
volatile uint8_t  stNum    = 0; // номер станции из eeprom

struct ds3231_calendar ds  = {0};// для хранения данных времени
uint8_t clnx; // мигает двоеточие
uint16_t disp_idle         = 0;
const typedef enum 
{
  ir_none    =      0,
  ir_power   =  0xA25D, // 41565 power 
  ir_mode    =  0x629D, // 25245 кнопка Mode
  ir_mute    =  0xE21D, // 57885 кнопка Mute 
  ir_play    =  0x22DD, //  8925 кнопка Play/pause
  ir_fbrev   =  0x02FD, //   765 кнопка >>
  ir_ffrev   =  0xC23D, // 49725 кнопка <<
  ir_eq      =  0xE01F, // 57375 кнопка EQ
  ir_voldown =  0xA857, // 43095 громкость -
  ir_volup   =  0x906F, // 36975 громкость +
  ir_zero    =  0x6897, // 26775 Zero
  ir_repeat  =  0x9867, // 39015 кнопка RPT
  ir_utf     =  0xB04F, // 45135 кнопка U/TF
  ir_one     =  0x30CF, // 12495 One
  ir_two     =  0x18E7, //  6375 Two
  ir_three   =  0x7A85, // 31365 Three
  ir_four    =  0x10EF, //  4335 Four
  ir_five    =  0x38C7, // 14535 Five
  ir_six     =  0x5AA5, // 23205 Six
  ir_seven   =  0x42BD, // 17085 Seven
  ir_eight   =  0x4AB5, // 19125 Eight
  ir_nine    =  0x52AD, // 21165 Nine
} ir_remote_keys;
ir_remote_keys irkey; //

void tim14_init() 
{
  RCC->APB1ENR |= RCC_APB1ENR_TIM14EN;
  TIM14->PSC = 47999;
  TIM14->ARR = 100; // 10 100 1000
  TIM14->DIER |= TIM_DIER_UIE;
  TIM14->CR1 |= TIM_CR1_CEN;
  //NVIC_SetPriority(TIM14_IRQn, 5); // ?
  NVIC_EnableIRQ(TIM14_IRQn);
}

void display_time()
{
  uint8_t a;
  ds3231get(&ds);
  (bcd2dec4(ds.second) & 1 ? (clnx = 0) : (clnx = 1));
  tm_disp_clock(bcd2dec4(ds.hour), bcd2dec4(ds.minute), clnx);
}

//
void show_radio_status(void)
{
  switch (dispmode) {
    case 0: tm_disp_div10freq(tuner.freq); break;    // отображает tuner.freq/10
    case 1: tm_disp_volume(tuner.volume, tuner.bass); break; // громкость
    case 3: tm_disp_channel(stNum); break;    // надо отобразить stNum
    case 4: tm_disp_oo(); break;    // отобразить save - ok
    case 5: tm_disp_chsaved(stNum); break; // tm_disp_chasaved
    case 6: tm_disp_oops(); break; // cancel
    case 7: break;
  }
}

// вынести весь tuner в отдельный tuner.h!
void tunerVolDn(void)
{
  tuner.volume--;
  if (tuner.volume < 1) tuner.volume = 0;
  dispmode = 1; dispcntr = 15;
  tunerSetVolume(tuner.volume);
  show_radio_status();
}

void tunerVolUp(void)
{
  tuner.volume++;
  if (tuner.volume >= 16) tuner.volume = 16;
  dispmode = 1; dispcntr = 15;
  tunerSetVolume(tuner.volume);
  show_radio_status();
}

void tuner_load_stNum(uint8_t station)
{
 stNum = station;
 eeprom_load_tuner_station(stNum);
 dispmode = 3; dispcntr = 15;
 show_radio_status();
}


void tuner_init_defs (void)
{
  // 
  tuner.step1 = 10; 
  tuner.step2 =  1;  
  tuner.fMin = 7600; //RDA5807_MIN_FREQ;  
  tuner.fMax = 10800;  tuner.freq =  9880;
  tuner.freq = 9880;
  tuner.volume = 4;
  last_freq = tuner.freq;
  last_volume = tuner.volume;
  rda580xInit();
  tunerSetFreq();
  tunerSetVolume(tuner.volume);
}

void hardware_init(void)
{
  rcc_sysclockinit();
  systick_init();
  SysTick_Config(FCPU);
  gpio_init();
  _delay_ms(1000);
  _delay_ms(100);
  zi2c_init(); 
  rda580xInit();
  ik_tim_init();
#ifdef USART_ON
  usart_init();
#endif
  eeprom_load_tuner_data();
  //tuner_init_defs();
  last_freq = tuner.freq;
  last_volume = tuner.volume;
  tm_init();
  tm_disp_clear();
  tim14_init();
  dispmode = 1; dispcntr = 40;
  tunerSetFreq();
  _delay_ms(60000);
  tunerSetVolume(tuner.volume);
  ds3231init();
  tunerSetFreq();
  ispowerup = 0; 
  tunerPowerOff();
}

void show2uart_ds3231()
{
  ds3231get(&ds);
  usart_puts("H: ");
  usart_put_u08(bcd2dec4(ds.hour));
  usart_puts(" :M: ");
  usart_put_u08(bcd2dec4(ds.minute));
  usart_puts(" :sec: ");
  usart_put_u08(bcd2dec(ds.second));
  usart_putc('\n');
}

void manual_set_date_time()
{ 
  ds.second   = dec2bcd4(0);
  ds.minute   = dec2bcd4(52);
  ds.hour     = dec2bcd4(19);
  ds.dayMonth = dec2bcd4(28);
  ds.month    = dec2bcd4( 3);
  ds.year     = dec2bcd4(23);
  ds3231set(&ds);
  usart_puts("Set OK!!: ");
  usart_putc('\n');
}

void save_channel_menu() 
{
  tm_disp_chsaved(stNum);
  ik_out = 1;
  while (ik_out) { // Сохранить да/нет +/-
    pin_set(GPIOA, 1<<15);
    switch (ik_out) 
    {
      case ir_volup  : stNum++; if (stNum > 24) stNum = 24; ik_out = 2; tm_disp_chsaved(stNum); break;
      case ir_voldown: stNum--; if (stNum > 24) stNum =  0; ik_out = 2; tm_disp_chsaved(stNum); break;
      case ir_play   : eeprom_save_tuner_station(stNum); dispmode = 4; dispcntr = 20; ik_out = 0; break;
      case ir_eq: case ir_zero: case ir_repeat:
                       dispmode = 6; dispcntr = 30; ik_out = 0; break;
    }
  }
  _delay_ms(300);
  show_radio_status();
  pin_clear(GPIOA, 1<<15);
}

void setup_time()
{
  uint8_t th, tm;
  ds3231get(&ds);
  th = bcd2dec4(ds.hour); tm = bcd2dec4(ds.minute);
  tm_disp_clock(bcd2dec4(ds.hour), bcd2dec4(ds.minute), 1);
  ik_out = 1;
  while (ik_out) {
  pin_set(GPIOA, 1<<15);
  switch (ik_out) 
  {
    case ir_volup  : tm++; if (tm > 59) tm = 0; ik_out = 2; 
                     ds.minute = dec2bcd4(tm); tm_disp_clock(bcd2dec4(ds.hour), bcd2dec4(ds.minute), 1); 
                     break;
    case ir_voldown: tm--; if (tm > 59) tm = 59; ik_out = 2;
                     ds.minute = dec2bcd4(tm); tm_disp_clock(bcd2dec4(ds.hour), bcd2dec4(ds.minute), 1);
                     break;
    case ir_ffrev  : th++; if (th > 23) th = 0; ik_out = 2; 
                     ds.hour = dec2bcd4(th); tm_disp_clock(bcd2dec4(ds.hour), bcd2dec4(ds.minute), 1);
                     break;
    case ir_fbrev  : th--; if (th > 23) th = 23; ik_out = 2;
                     ds.hour = dec2bcd4(th); tm_disp_clock(bcd2dec4(ds.hour), bcd2dec4(ds.minute), 1);
                     break;
    case ir_play   : ds.hour = dec2bcd4(th); ds.minute = dec2bcd4(tm); ds.second = 0; 
                     ds3231set(&ds);
                     dispmode = 4; dispcntr = 30; ik_out = 0; _delay_ms(1000);
                     break;
    case ir_eq: case ir_zero: case ir_repeat:
                     dispmode = 6; dispcntr = 30; ik_out = 0;
                     break;
  }
  pin_clear(GPIOA, 1<<15);
  }
}



// основное:
void check_IR_command()
{
  ikp = ttms - ikp; // замер паузы меду посылками
  if (ikp < 400) ik_out_repeatf++; else ik_out_repeatf = 0;
  if (ik_out_repeatf > 3) ik_out_repeatf = 3;
  

  if (ik_out_repeatf == 3) { // повторные срабатывания с паузой
    switch (ik_out) {
      case ir_power  : break;
      case ir_voldown: tunerVolDn(); break;
      case ir_volup  : tunerVolUp(); break;
      case ir_ffrev  : tunerChangeFreq( 10); dispmode = 0; dispcntr =  2; break;
      case ir_fbrev  : tunerChangeFreq(-10); dispmode = 0; dispcntr =  2; break;
      case ir_zero   : break;//dispmode = 5; dispcntr = 30; eeprom_save_tuner_station(stNum); break;
      case ir_mode   : save_channel_menu(); break;
      case ir_utf    : setup_time(); break;
    } 
  } // if удержания кнопки


  if (ik_out_repeatf == 1) { // одиночное срабатывание (медленные повторы)
    switch (ik_out) {
      case ir_power  : if (ispowerup) {tunerPowerOff();} else {tunerPowerOn();} break;
      case ir_mode   : if (dispmode == 7) dispmode = 0; else { dispmode = 7; display_time(); } break;
      case ir_mute   : tunerSetMute(tuner.mute = !tuner.mute); break;
      case ir_play   : break;
      case ir_ffrev  : tunerChangeFreq( 10); dispmode = 0; dispcntr =  2; break; 
      case ir_fbrev  : tunerChangeFreq(-10); dispmode = 0; dispcntr =  2; break;
      case ir_eq     : tunerSetBass(tuner.bass = !tuner.bass); dispmode = 1; dispcntr = 25; break;
      case ir_voldown: tunerVolDn(); break;
      case ir_volup  : tunerVolUp(); break;
      case ir_zero   : tuner_load_stNum( 0); break;
      case ir_repeat : show2uart_ds3231(); break; //scani2c2usart(); break;
      case ir_utf    : break; //dispmode = 4; dispcntr = 20; eeprom_save_tuner_data(); break; // save tuner data
      case ir_one    : tuner_load_stNum( 1); break;
      case ir_two    : tuner_load_stNum( 2); break;
      case ir_three  : tuner_load_stNum( 3); break;
      case ir_four   : tuner_load_stNum( 4); break;
      case ir_five   : tuner_load_stNum( 5); break;
      case ir_six    : tuner_load_stNum( 6); break;
      case ir_seven  : tuner_load_stNum( 7); break;
      case ir_eight  : tuner_load_stNum( 8); break;
      case ir_nine   : tuner_load_stNum( 9); break; //manual_set_date_time(); break; //scani2c2usart(); break; 

    } // switch (ik_out)
  } // if проверки одиночного повтора 
#ifdef USART_ON
  //usart_puts("Jack pot!: "); usart_put_hex(ik_out, 3); usart_putc('\n');
  //usart_puts("Jack pot!: "); usart_putU16(ik_out); usart_putc('\n');
#endif
  ik_out = 0; //
  ikp = ttms;  // выравнили
  disp_idle = 0;
} // void check_IR_command() {}

void systick_blink_pb3(uint16_t freq) 
{
  if (_pb3led > ttms || ttms - _pb3led > freq) {
    //pin_toggle(GPIOB, 1<<3);
    disp_idle++;
    if (disp_idle > 10 ) { display_time(); disp_idle = 10; }
    _pb3led = ttms;
#ifdef USART_ON
    //usart_putU16(_pb3led); usart_putc('\n');
#endif
  }
}

//##################################################################//
//##################################################################//
int main(void) 
{
  hardware_init();
//##################################################################//
  do 
  {
    systick_blink_pb3(999);
    if (ik_out) check_IR_command();
    // pin_toggle(GPIOB, 1<<3);
    // bla bla bla
    if (dispmode == 7) display_time();
    // bla bla bla
  } while (1); // do {}
}
//##################################################################//
//##################################################################//
// прерывание системного тикера
void SysTick_Handler(void) 
{
  ++ttms; 
}

// прерывание под вопросом
void TIM14_IRQHandler() 
{
  TIM14->SR &= ~TIM_SR_UIF;
  //GPIOB->ODR ^= (1<<3);   // проверить светодиод _pb3led

  if (dispcntr > 0 ) dispcntr--;
  if (last_dispmode != dispmode) show_radio_status();
  if (dispcntr == 1) { dispmode = 0; dispcntr = 0; show_radio_status(); }
  last_dispmode = dispmode;
}